/*
|
|      $Id: tiffdecex.cpp,v 1.1 1999/10/19 21:30:42 Ulrich von Zadow Exp $
|      TIFF Decoder Class
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"
#include "tiffdec.h"
#include "tiffdecex.h"
#include "except.h"

extern "C"
{
#include "tif_msrc.h"
}

#ifdef _WINDOWS
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
IMPLEMENT_DYNAMIC (CTIFFDecoderEx, CTIFFDecoder);
#endif

/////////////////////////////////////////////////////////////////////
//  helper class to setup libtiff callbacks

class _tiff_helper
{
public:
  _tiff_helper()
  {
   	// Set up function pointers to error handling routines.
    TIFFSetErrorHandler (CTIFFDecoder::Win32ErrorHandler);
    TIFFSetWarningHandler (CTIFFDecoder::Win32WarningHandler);
  }

  // We could restore the internal (default) libtiff handlers,
  // but that is not generally useful
  ~_tiff_helper() {}
};

static _tiff_helper _the_tiff_helper_singleton;


/////////////////////////////////////////////////////////////////////
// slightly more capable decoder...

CTIFFDecoderEx::CTIFFDecoderEx() : m_TiffToken(0)
{}


CTIFFDecoderEx::~CTIFFDecoderEx()
{ Dissociate(); }


BOOL CTIFFDecoderEx::Associate( CDataSource* pDataSrc )
{
  m_TiffToken = TIFFOpenMem (pDataSrc->ReadEverything(),
						   pDataSrc->GetFileSize(),
						   NULL);
  return m_TiffToken != 0;
}


void CTIFFDecoderEx::Dissociate()
{
  if ( m_TiffToken ) 
  { TIFFClose( m_TiffToken ); m_TiffToken = 0; }
}


void CTIFFDecoderEx::DoDecode
    ( CBmp * pBmp,
      RGBAPIXEL ** ppPal,
      int * pDestBPP,
      CDataSource * pDataSrc
    )
{
  ASSERT( m_TiffToken );

  // call base version on associated data source
  CTIFFDecoder::DoTiffDecode( 
    pBmp, 
    ppPal, 
    pDestBPP, 
    pDataSrc,
    m_TiffToken 
  );
}

// It'd be nicer to define a bunch of type-safe functions like:
//    uint32  CTIFFDecoderEx::GetImageLength();
//    CString CTIFFDecoderEx::GetImageDescription();
int CTIFFDecoderEx::GetField( int tag_id, ... )
{
  va_list marker;
  int retv;

  va_start( marker, tag_id );     /* Initialize variable arguments. */
  retv = TIFFVGetFieldDefaulted( m_TiffToken, (ttag_t) tag_id, marker );
  va_end( marker );              /* Reset variable arguments.      */

  return retv;
}
/*
/--------------------------------------------------------------------
|
|      $Log: tiffdecex.cpp,v $
|      Revision 1.1  1999/10/19 21:30:42  Ulrich von Zadow
|      B. Delmee - Initial revision
|
|
--------------------------------------------------------------------
*/
