/*
/--------------------------------------------------------------------
|
|      $Id: tiffencex.cpp,v 1.1 1999/10/19 21:30:42 Ulrich von Zadow Exp $
|      TIFF Encoder Class
|
|      TIFF file encoder. Uses LIBTIFF to do the actual conversion.
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"

// Not quite ready for prime-time; bdelmee; 2/99
// I mostly needed monochrome TIFF writing, which seems robust enough.
// Palette or RGB writing seems to work but has not been sufficiently tested.

#include <stdarg.h>
#include "tiffenc.h"
#include "tiffencex.h"
#include "bitmap.h"
#include "except.h"

extern "C"
{
#include "tif_msrc.h"
}

#ifdef _WINDOWS
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
IMPLEMENT_DYNAMIC (CTIFFEncoderEx, CTIFFEncoder);
#endif


/////////////////////////////////////////////////////////////////////
// more format-specific encoder


CTIFFEncoderEx::CTIFFEncoderEx() : m_TiffToken(0)
{}


CTIFFEncoderEx::~CTIFFEncoderEx()
{ Dissociate(); }


BOOL CTIFFEncoderEx::Associate( CDataSink* pDataSnk )
{
  m_TiffToken = TIFFOpenMem (pDataSnk->m_pStartData,
						     pDataSnk->m_nMaxFileSize,
						     &(pDataSnk->m_nCurPos));
  return m_TiffToken != 0;
}


void CTIFFEncoderEx::Dissociate()
{
  if ( m_TiffToken ) 
  { TIFFClose( m_TiffToken ); m_TiffToken = 0; }
}


void CTIFFEncoderEx::DoEncode
    ( CBmp * pBmp,
      CDataSink* /* pDataSnk */
    )
{
  ASSERT( m_TiffToken );
  // call base version on open tiff descriptor
  CTIFFEncoder::DoTiffEncode( pBmp, m_TiffToken );
}


// The following two calls make their base class equivalent usable,
// without requiring the user to know about the libtiff internals (TIFF*)
int CTIFFEncoderEx::SetBaseTags( CBmp* pBmp )
{
  return CTIFFEncoder::SetBaseTags( m_TiffToken, pBmp );
}


int CTIFFEncoderEx::SetField( int tag_id, ... )
{
  int retv;
  va_list marker;

  va_start( marker, tag_id );     /* Initialize variable arguments. */
  retv = TIFFVSetField( m_TiffToken, tag_id, marker );
  va_end( marker );              /* Reset variable arguments.      */

  return retv;
}
/*
/--------------------------------------------------------------------
|
|      $Log: tiffencex.cpp,v $
|      Revision 1.1  1999/10/19 21:30:42  Ulrich von Zadow
|      B. Delmee - Initial revision
|
|
--------------------------------------------------------------------
*/
