/*
/--------------------------------------------------------------------
|
|      $Id$
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdafx.h"
#include <afxole.h>
#include "PaintX.h"
#include "PictureDecoder.h"

const int INTERNALBPP = 0;

/////////////////////////////////////////////////////////////////////////////
// CPictureDecoder

STDMETHODIMP CPictureDecoder::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IPictureDecoder,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPictureDecoder::LoadPicture(BSTR Filename, IUnknown ** ppUnk)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

  HRESULT hResult=S_OK;

	CWinBmp * pWinBmp=new CWinBmp;

	LPSTREAM pStream=NULL;


  // Big try..catch wrapper to catch any exceptions
	// before they reach COM.
	try
	{
		m_AnyPicDecoder.MakeBmpFromFile((LPCSTR)CString(Filename), pWinBmp,INTERNALBPP);
		pStream = SaveBitmapToGMemStream(pWinBmp);
		// save memory - we don't need the CWinBmp anymore
		delete pWinBmp;
		pWinBmp = NULL;

		// Create a COM Picture Object from this stream
    hResult = OleLoadPicture(pStream,0,FALSE, IID_IPictureDisp, (void**)ppUnk);

		// Get rid of the Stream and its Global Memory
    pStream->Release();
	  pStream=NULL;

		// If the picture is ok, then call IPicture::PictureChanged
		if (SUCCEEDED(hResult))
			hResult = SignalPictureChanged(*ppUnk);
	}

	catch (CTextException e)
  {
      // Handle PaintLib exceptions
      hResult = this->CreateErrorInfo(e, IID_IPictureDecoder);	
	}

	catch (CException* e)
  {
    // Handle all other exceptions
    hResult = this->CreateErrorInfo(e, IID_IPictureDecoder);	
	}

	// Get rid of the bitmap, if it exists
	if (pWinBmp)
	{
		delete pWinBmp;
		pWinBmp = NULL;
	}
	// Get rid of the stream, if it exists
	if (pStream)
	{
		pStream->Release();
		pStream = NULL;
	}
  // We have an error condition 
	// AND an (allocated) Picture, so we must 
	// get rid of the Picture Object
	if (FAILED(hResult) && *ppUnk)
	{
		(*ppUnk)->Release();
		(*ppUnk)=NULL;
	}
	return hResult;
}


LPSTREAM CPictureDecoder::SaveBitmapToGMemStream(CWinBmp* pWinBmp)
{
  COleStreamFile oleStreamFile;
  CFileException fileException;
  LPSTREAM lpStream = NULL;
  try
  {
	  // Create a CFile-derived COleStreamFile in Memory
    if (!oleStreamFile.CreateMemoryStream(&fileException))
      AfxThrowFileException(fileException.m_cause,
                            fileException.m_lOsError,
                            fileException.m_strFileName);
 
    // Write the bitmap to the OleStreamFile
    pWinBmp->SaveAsBmp(oleStreamFile);

    // Detach the oleStreamFile and retrieve the IStream
    lpStream = oleStreamFile.Detach();

  }
  catch (...)
  {
	  if (lpStream)
	  {
		  lpStream->Release();
		  lpStream = NULL;
	  }
	  throw; // re-throw exception
  }

  // Seek to beginning of stream
  LARGE_INTEGER li_NULL;
  li_NULL.LowPart = 0;
  li_NULL.HighPart = 0;
  (lpStream)->Seek(li_NULL,STREAM_SEEK_SET,NULL);

  return lpStream;
}

HRESULT CPictureDecoder::SignalPictureChanged(IUnknown * pUnknown)
{
  HRESULT hResult = S_OK;

   // Get Picture Interface (only the Picture Interface has the PictureChanged method)
  IPicture* pPicture;
  if(SUCCEEDED(pUnknown->QueryInterface(IID_IPicture, (LPVOID *) &pPicture))) 
  {
    hResult = pPicture->PictureChanged();
	  // release the reference 
	  pPicture->Release();
  };

  return hResult;
}

HRESULT CPictureDecoder::CreateErrorInfo(CTextException  TextException,REFIID riidSource)
{ 
  // create an error info object 
  ICreateErrorInfo * pcerrinfo; 
  HRESULT hr = ::CreateErrorInfo(&pcerrinfo); 
  // if we succeeded 
  if (SUCCEEDED(hr)) 
  { 
    int ErrCode = TextException.GetCode();

    switch(ErrCode)
	  {
      // Map PaintLib Error Codes to Win32 Codes to COM HRESULT Codes
	    case ERR_PATH_NOT_FOUND:
		    hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_WIN32, ERROR_FILE_NOT_FOUND);
		    break;
      case ERR_FILE_NOT_FOUND:
	      hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_WIN32, ERROR_FILE_NOT_FOUND);
		    break;
	    case  ERR_ACCESS_DENIED:
		    hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_WIN32, ERROR_ACCESS_DENIED);
		    break;
	    // Map from PaintLib Error Codes to COM Interface-Independent Error Codes
	    case ERR_NO_MEMORY:
        hr = E_OUTOFMEMORY;
		    break;
	    // Map From PaintLib Error Codes to COM Interface-Dependent HRESULT Codes
	    case ERR_INTERNAL:
	    case ERR_FORMAT_NOT_SUPPORTED:
	    case ERR_WRONG_SIGNATURE:
	    case ERR_FORMAT_UNKNOWN:
	    case ERR_UNKNOWN_FILE_TYPE:
	    case ERR_END_OF_FILE:
	    default:
        hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, ErrCode);
	  }
	
	// Set up ErrInfo object 
	pcerrinfo->SetGUID(riidSource); 
	CString cstrFileName(AfxGetAppName()); 
	pcerrinfo->SetSource(cstrFileName.AllocSysString()); 
	CString cstrDescription(TextException);
	pcerrinfo->SetDescription(cstrDescription.AllocSysString());

	// QI for the IErrorInfo interface 
	IErrorInfo * perrinfo; 
	if(SUCCEEDED(pcerrinfo->QueryInterface(IID_IErrorInfo, (LPVOID *) &perrinfo))) 
	{ 
		// set the error info object 
		::SetErrorInfo(0, perrinfo); 
		// release the reference 
		perrinfo->Release(); 
	} 
	// release the reference 
	pcerrinfo->Release(); 
  } 
  // return the error value 
  return hr; 
} 
  

HRESULT CPictureDecoder::CreateErrorInfo(CException * pAnyException, REFIID riidSource) 
{ 
  ASSERT_VALID(pAnyException); 
  // create an error info object 
  ICreateErrorInfo * pcerrinfo; 
  HRESULT hr = ::CreateErrorInfo(&pcerrinfo); 
  // if we succeeded 
  if(SUCCEEDED(hr)) 
  { 
    // dispatch exception? 
    if(pAnyException->IsKindOf(RUNTIME_CLASS(COleDispatchException))) 
	{ 
		// specific IDispatch style exception 
		COleDispatchException * e = (COleDispatchException *) pAnyException; 
		// set the return value to the error 
		hr = e->m_scError; 
		// Set up ErrInfo object 
		pcerrinfo->SetGUID(riidSource); 
		pcerrinfo->SetDescription(e->m_strDescription.AllocSysString()); 
		pcerrinfo->SetHelpContext(e->m_dwHelpContext); 
		pcerrinfo->SetHelpFile(e->m_strHelpFile.AllocSysString()); 
		pcerrinfo->SetSource(e->m_strSource.AllocSysString()); 
	} 
	else if (pAnyException->IsKindOf(RUNTIME_CLASS(CMemoryException))) 
	{ 
		// failed memory allocation 
		hr = E_OUTOFMEMORY; 
		// Set up ErrInfo object 
		pcerrinfo->SetGUID(riidSource); 
		CString cstrFileName(AfxGetAppName()); 
		pcerrinfo->SetSource(cstrFileName.AllocSysString()); 
	} 
	else 
	{ 
		// other unknown/uncommon error 
		hr = E_UNEXPECTED; 
		// Set up ErrInfo object 
		pcerrinfo->SetGUID(riidSource); 
		CString cstrFileName(AfxGetAppName()); 
		pcerrinfo->SetSource(cstrFileName.AllocSysString()); 
	} 
	// QI for the IErrorInfo interface 
	IErrorInfo * perrinfo; 
	if(SUCCEEDED(pcerrinfo->QueryInterface(IID_IErrorInfo, (LPVOID *) &perrinfo))) 
	{ 
		// set the error info object 
		::SetErrorInfo(0, perrinfo); 
		// release the reference 
		perrinfo->Release(); 
	} 
	// release the reference 
	pcerrinfo->Release(); 
  } 
  // delete the exception 
  pAnyException->Delete(); 
  // return the error value 
  return hr; 
} 

STDMETHODIMP CPictureDecoder::LoadResPicture(int ResourceID, IUnknown ** ppUnk)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

    HRESULT hResult=S_OK;

	CWinBmp * pWinBmp=new CWinBmp;
	LPSTREAM pStream=NULL;

    // Big try..catch wrapper to catch any exceptions
	// before they reach COM.
	try
	{
		m_AnyPicDecoder.MakeBmpFromResource(ResourceID, pWinBmp, INTERNALBPP);
		pStream = SaveBitmapToGMemStream(pWinBmp);
		// save memory - we don't need the CWinBmp anymore
		delete pWinBmp;
		pWinBmp = NULL;

		// Create a COM Picture Object from this stream
    hResult = OleLoadPicture(pStream,0,FALSE, IID_IPictureDisp, (void**)ppUnk);

		// Get rid of the Stream and its Global Memory
    pStream->Release();
	  pStream=NULL;

		// If the picture is ok, then call IPicture::PictureChanged
		if (SUCCEEDED(hResult))
			hResult = SignalPictureChanged(*ppUnk);
	}

	catch (CTextException e)
  {
    // Handle PaintLib exceptions
    hResult = this->CreateErrorInfo(e, IID_IPictureDecoder);	
	}

	catch (CException* e)
  {
    // Handle all other exceptions
    hResult = this->CreateErrorInfo(e, IID_IPictureDecoder);	
	}

	// Get rid of the bitmap, if it exists
	if (pWinBmp)
	{
		delete pWinBmp;
		pWinBmp = NULL;
	}
	// Get rid of the stream, if it exists
	if (pStream)
	{
		pStream->Release();
		pStream = NULL;
	}
    // We have an error condition 
	// AND a (allocated) Picture, so we must 
	// get rid of the Picture Object
	if (FAILED(hResult) && *ppUnk)
	{
		(*ppUnk)->Release();
		(*ppUnk)=NULL;
	}
	return hResult;
}

/*
/--------------------------------------------------------------------
|
|      $Log$
|
--------------------------------------------------------------------
*/
