/*
/--------------------------------------------------------------------
|
|      $Id: Doc.cpp,v 1.5 1999/11/08 22:15:05 Ulrich von Zadow Exp $
|      DOC.CPP    Document class implementation (where the bitmaps live)
|
|      Copyright (c) 1998 Bernard Delme
|
\--------------------------------------------------------------------
*/

#include "stdafx.h"
#include "piclook.h"
#include "doc.h"

#include "MainFrm.h"              // Jo Hagelberg 15.4.99

#include "tiffencex.h"
#include "jpegenc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPLViewerDoc

IMPLEMENT_DYNCREATE(CPLViewerDoc, CDocument)

BEGIN_MESSAGE_MAP(CPLViewerDoc, CDocument)
	//{{AFX_MSG_MAP(CPLViewerDoc)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPLViewerDoc construction/destruction

CPLViewerDoc::CPLViewerDoc()
{
	m_pDib = 0;
}

CPLViewerDoc::~CPLViewerDoc()
{
	DeleteContents();
}

BOOL CPLViewerDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
  // recycle document
  DeleteContents();

	m_pDib = new CBmpClass;

    // have PaintLib perform I/O instead of Serialization
    BOOL bRet = TRUE;
	try {
    // Jo Hagelberg 15.4.99: added "this" for progress notification
#ifdef _IGNORES_PALETTE
		thisApp.GetDecoder()->MakeBmpFromFile(lpszPathName, m_pDib, 32, this);
#else
		thisApp.GetDecoder()->MakeBmpFromFile(lpszPathName, m_pDib, 0, this);
#endif
	}
	catch (CTextException e)
	{
		AfxMessageBox(e);
	    DeleteContents();
	    bRet = FALSE;
	}

    if (bRet)
	{
		SetPathName(lpszPathName);
		SetModifiedFlag(FALSE);     // start off with unmodified
		char *cp = strrchr(lpszPathName,'\\');
		if (cp) // establish this image's directory as our new wd
		{
			*cp = '\0';
			SetCurrentDirectory(lpszPathName);
			// pretend we did not touch that string
			*cp = '\\';
		}
	}

    return bRet;		// Belgian resolution enhancement technology :-)
}

// Jo Hagelberg 15.4.99: copied OnProgress from DibStatic
// (bad design, anyway, this is just for testing ;-)
void CPLViewerDoc::OnProgress 
    ( double Part
    )
{ 
	//TRACE ("V %.4f\n", Part);
	// Jo Hagelberg 13.4.99
	// added real progress info
	UINT ipart = UINT( Part * 100.0);
	((CMainFrame*)AfxGetMainWnd())->SetProgressInfo(ipart);
}

void CPLViewerDoc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (m_pDib != NULL);
}

void CPLViewerDoc::OnFileSave() 
{
  CFileDialog	SaveDlg (FALSE, ".tif", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                       "JPEG Files (*.jpg)|*.jpg|TIFF Files (*.tif)|*.tif",
                       AfxGetMainWnd());
  SaveDlg.DoModal();
  CString sFName = SaveDlg.GetPathName();
  CString sExt = SaveDlg.GetFileExt();
  sExt.MakeLower();
  if (sExt == "jpg")
  {
    CJPEGEncoder Encoder;
    Encoder.MakeFileFromBmp(sFName, m_pDib);
  }
  else
  {
    if (sExt == "tif")
    {
      CTIFFEncoder Encoder;
      Encoder.MakeFileFromBmp(sFName, m_pDib);
    }
    else
    {
      // Extension must be either tif or jpg. If not, something is wrong
      // in the user interface code.
      ASSERT (FALSE);
    }
  }
}

/*
BOOL CPLViewerDoc::OnSaveDocument( LPCTSTR lpszPathName )
{
	if (m_pDib)
	{
		m_pDib->SaveAsBmp(lpszPathName);
		return TRUE;
	}
	else
		return FALSE;
}
*/

//////////////////
// Delete contents of doc: delete the DIB
//
void CPLViewerDoc::DeleteContents()
{
	if (m_pDib) delete m_pDib;
	m_pDib = 0;
}

/////////////////////////////////////////////////////////////////////////////
// CPLViewerDoc diagnostics

#ifdef _DEBUG
void CPLViewerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CPLViewerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}

#endif //_DEBUG
/*
/--------------------------------------------------------------------
|
|      $Log: Doc.cpp,v $
|      Revision 1.5  1999/11/08 22:15:05  Ulrich von Zadow
|      Added File/SaveAs
|
|
\--------------------------------------------------------------------
*/
