/*
/--------------------------------------------------------------------
|
|      $Id: DocMan.cpp,v 1.5 1999/10/21 17:43:19 Ulrich von Zadow Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\-------------------------------------------------------------------
*/

/*
 *	Customised document manager (for MFC doc/view framework)
 *	This code copied from MFC sources, except that we use our derived
 *	CFileDialog for graphic files previewing and adapt file filtering
 *  bdelmee; 4/99
 */

#include "stdafx.h"
#include "DibVfw.h"
#include "DibCtl.h"
#include "Preview.h"
#include "DocMan.h"

// see what graphic formats are compiled in paintlib
#include "config.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static void AppendFileType(CString& filter, CString& catfilter,
                           LPCTSTR desc, LPCTSTR extn)
{
	filter += desc;
	filter += (TCHAR)'\0';
	filter += extn;
	filter += (TCHAR)'\0';
    if (! catfilter.IsEmpty())
        catfilter += ';';
    catfilter += extn;
}


/////////////////////////////////////////////////////////////////////////////
// CDocManagerEx

IMPLEMENT_DYNAMIC(CDocManagerEx, CDocManager)

BOOL CDocManagerEx::DoPromptFileName(CString& fileName, UINT nIDSTitle, DWORD lFlags, BOOL bOpenFileDialog, CDocTemplate* pTemplate)
{
	CPreviewFileDlg dlgFile(bOpenFileDialog);

	CString title;
	VERIFY(title.LoadString(nIDSTitle));

	// our program is but a viewer, so let's use read-only mode
    dlgFile.m_ofn.Flags |= lFlags | OFN_READONLY;

	CString strFilter;
  CString strGrfFlt;

    // instead of using the template, let's build our filter
    // according to the graphic formats for which support was 
    // compiled in paintlib
#ifdef SUPPORT_BMP
  AppendFileType(strFilter,strGrfFlt,"BMP file","*.bmp;*.dib;*.rle");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_JPEG
  AppendFileType(strFilter,strGrfFlt,"JPEG file","*.jpg;*.jpeg;*.jif");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_PNG
  AppendFileType(strFilter,strGrfFlt,"PNG file","*.png");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_PICT
  AppendFileType(strFilter,strGrfFlt,"PICT file","*.pct");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_TIFF
  AppendFileType(strFilter,strGrfFlt,"TIFF file","*.tif;*.eps");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_TGA
  AppendFileType(strFilter,strGrfFlt,"TARGA file","*.tga");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_WEMF
  AppendFileType(strFilter,strGrfFlt,"Windows Metafile","*.wmf");
	dlgFile.m_ofn.nMaxCustFilter++;
  AppendFileType(strFilter,strGrfFlt,"Windows Enhanced Metafile","*.emf");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif
#ifdef SUPPORT_TGA
  AppendFileType(strFilter,strGrfFlt,"PCX file","*.pcx");
	dlgFile.m_ofn.nMaxCustFilter++;
#endif

  // add default filter
  AppendFileType(strFilter,strGrfFlt,
        "All supported graphic formats",(LPCTSTR) strGrfFlt);
	dlgFile.m_ofn.nMaxCustFilter++;
	dlgFile.m_ofn.nFilterIndex = dlgFile.m_ofn.nMaxCustFilter;
	
  // append the "*.*" all files filter
	CString allFilter;
	VERIFY(allFilter.LoadString(AFX_IDS_ALLFILTER));
  AppendFileType(strFilter,strGrfFlt,allFilter,"*.*");
  //	dlgFile.m_ofn.nMaxCustFilter++;

	dlgFile.m_ofn.lpstrFilter = strFilter;
	dlgFile.m_ofn.lpstrTitle = title;
	dlgFile.m_ofn.lpstrFile = fileName.GetBuffer(_MAX_PATH);

	BOOL bResult = dlgFile.DoModal() == IDOK ? TRUE : FALSE;
	fileName.ReleaseBuffer();
	return bResult;
}
/*
/--------------------------------------------------------------------
|
|      $Log: DocMan.cpp,v $
|      Revision 1.5  1999/10/21 17:43:19  Ulrich von Zadow
|      Added pcx support by Meng Bo.
|
|
\-------------------------------------------------------------------
*/
